/*
  Velleman_8056_16F84.h - Library for driving velleman 8056 8 relay board RS232 customized with a 16F84 pic for increased speed com.
  Created by Daniel CALLERAND, June 24, 2011.
  Released as is without any kind of guarantee except it works for me as i wanted to and i hope you find it useful. 
*/

#include "WProgram.h"
#include "Velleman_8056_16F84.h"


Velleman_8056_16F84::Velleman_8056_16F84(int rs232Pin, int speed)
{
	pinMode(rs232Pin, OUTPUT);
	_rs232Pin = rs232Pin;
	_comSpeed = speed;
	_bitPeriod = 1000000 / _comSpeed;


	digitalWrite(_rs232Pin, HIGH);
	delayMicroseconds( _bitPeriod); // if we were low this establishes the end


	_rptCmd = 2;
	_run = 0;
	_cmd = 0x00;
	_runCount = 1;
	runDelay = 250;
	
	_mems[0] = B00000001;
	_mems[1] = B00000010;
	_mems[2] = B00000100;
	_mems[3] = B00001000;
	_mems[4] = B00010000;
	_mems[5] = B00100000;
	_mems[6] = B01000000;
	_mems[7] = B10000000;
	

	
	
  }

void Velleman_8056_16F84::emergencyStop()
{
	_cmd = 0x00;
	sendCommand();
  
}



void Velleman_8056_16F84::setRelay(byte relay)
{
	byte mask;

	mask  = 1;

	for(int i = 1; i < relay; i++) {
		mask *= 2;
	}

	_cmd = _cmd | mask;

	sendCommand();
}

void Velleman_8056_16F84::clearRelay(unsigned char relay)
{
	byte mask;

	mask  = 1;

	for(int i = 1; i < relay; i++) {
		mask *= 2;
	}

	mask = 0xFF - mask;

	_cmd = _cmd & mask;

	sendCommand();
  
}

void Velleman_8056_16F84::toggleRelay(unsigned char relay)
{
	byte mask;
	mask  = 1;
	for(int i = 1; i < relay; i++) {
		mask *= 2;
	}

	_cmd = _cmd ^ mask;

	sendCommand();
}

void Velleman_8056_16F84::store(int mem) {
	_mems[mem -1] = getCmd();
}
 
void Velleman_8056_16F84::recall(int mem) {
	sendByte(_mems[mem -1]);
}

void Velleman_8056_16F84::run() {
	_run = true;
	_runCount = 1;
}
void Velleman_8056_16F84::stopRunning() {
	_run = false;
}

bool Velleman_8056_16F84::isRunning() {
	return _run;
}

void Velleman_8056_16F84::next() {
	Serial.print(_runCount);
	Serial.print(" : ");
	Serial.println(_mems[_runCount-1],BIN);
	sendByte(_mems[_runCount-1]);
	_runCount++;
	if(_runCount == 9) {
		_runCount = 1;
	}
//	Serial.println(getCmd(),BIN);
		
}


void Velleman_8056_16F84::sendByte(unsigned char statut_byte)
{
//	checkSum();
	_cmd = statut_byte;
	sendCommand();
  
}

void Velleman_8056_16F84::sendCommand()
{
 
	//for (int j = 0; j < _rptCmd; j++) {
		print((uint8_t) _cmd);
	//}

}

void Velleman_8056_16F84::incRpt()
{
	if(runDelay < 1000)
		runDelay ++;
}

void Velleman_8056_16F84::decRpt()
{
	if(runDelay > 10)
		runDelay --;
}

void Velleman_8056_16F84::setRpt(int rpt)
{
	_rptCmd = rpt;
}


void Velleman_8056_16F84::print(uint8_t b)
{
    
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_rs232Pin , LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_rs232Pin ,HIGH); // send 1
    }
    else{
      digitalWrite(_rs232Pin ,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_rs232Pin , HIGH);
  delayMicroseconds(bitDelay);
}

byte Velleman_8056_16F84::getCmd() {
	return _cmd;
}












