/*
  Velleman_8056_16F84.h - Library for driving velleman 8056 8 relay board RS232 customized with a 16F84 pic for increased speed com.
  Created by Daniel CALLERAND, June 24, 2011.
  Released as is without any kind of guarantee except it works for me as i wanted to and i hope you find it useful. 
*/

#include "WProgram.h"
#include "Velleman_8056.h"


Velleman_8056::Velleman_8056(int rs232Pin, unsigned char address)
{
  _rs232Pin = rs232Pin;
  _address = address;
  _comSpeed = 2400;
  _bitPeriod = 1000000 / _comSpeed;
  _rptCmd = 2;

}

Velleman_8056::Velleman_8056(int rs232Pin, unsigned char address, int rpt)
{
  _rs232Pin = rs232Pin;
  _address = address;
  _comSpeed = 2400;
  _bitPeriod = 1000000 / _comSpeed;
  _rptCmd = rpt;

}

void Velleman_8056::emergencyStop()
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'E';
	_cmd[3] = 0x00;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::displayAddress()
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'D';
	_cmd[3] = 0x00;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::setAddress(unsigned char address)
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'A';
	_cmd[3] = address;
	checkSum();
	sendCommand();
	_address = address;
  
}

void Velleman_8056::forceAllCardAddr()
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'F';
	_cmd[3] = 0x00;
	checkSum();
	sendCommand();
	_address = 1;
  
}

void Velleman_8056::setRelay(unsigned char relay)
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'S';
	_cmd[3] = relay;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::clearRelay(unsigned char relay)
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'C';
	_cmd[3] = relay;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::toggleRelay(unsigned char relay)
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'T';
	_cmd[3] = relay;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::sendByte(unsigned char statut_byte)
{
	_cmd[0] = 13;
	_cmd[1] = _address;
	_cmd[2] = 'B';
	_cmd[3] = statut_byte;
	checkSum();
	sendCommand();
  
}

void Velleman_8056::checkSum()
{
  unsigned char checksum ;

   _cmd[4] = ~((13 + _cmd[1] + _cmd[2] + _cmd[3]) & 0xFF) + 1 ;
   
}

void Velleman_8056::sendCommand()
{
 for (int j = 0; j < _rptCmd; j++) {
	 for (int i = 0; i < 5; i++) {
  		print(_cmd[i],BYTE);
 	}
 }
}

void Velleman_8056::incRpt()
{
 if(_rptCmd < 100)
	_rptCmd ++;
}

void Velleman_8056::decRpt()
{
 if(_rptCmd > 1)
	_rptCmd --;
}

void Velleman_8056::setRpt(int rpt)
{
	_rptCmd = rpt;
}


void Velleman_8056::print(uint8_t b)
{
    
  int bitDelay = _bitPeriod - clockCyclesToMicroseconds(50); // a digitalWrite is about 50 cycles
  byte mask;

  digitalWrite(_rs232Pin , LOW);
  delayMicroseconds(bitDelay);

  for (mask = 0x01; mask; mask <<= 1) {
    if (b & mask){ // choose bit
      digitalWrite(_rs232Pin ,HIGH); // send 1
    }
    else{
      digitalWrite(_rs232Pin ,LOW); // send 1
    }
    delayMicroseconds(bitDelay);
  }

  digitalWrite(_rs232Pin , HIGH);
  delayMicroseconds(bitDelay);
}

void Velleman_8056::print(const char *s)
{
  while (*s)
    print(*s++);
}

void Velleman_8056::print(char c)
{
  print((uint8_t) c);
}

void Velleman_8056::print(int n)
{
  print((long) n);
}

void Velleman_8056::print(unsigned int n)
{
  print((unsigned long) n);
}

void Velleman_8056::print(long n)
{
  if (n < 0) {
    print('-');
    n = -n;
  }
  printNumber(n, 10);
}

void Velleman_8056::print(unsigned long n)
{
  printNumber(n, 10);
}

void Velleman_8056::print(long n, int base)
{
  if (base == 0)
    print((char) n);
  else if (base == 10)
    print(n);
  else
    printNumber(n, base);
}

void Velleman_8056::println(void)
{
  print('\r');
  print('\n');  
}

void Velleman_8056::println(char c)
{
  print(c);
  println();  
}

void Velleman_8056::println(const char c[])
{
  print(c);
  println();
}

void Velleman_8056::println(uint8_t b)
{
  print(b);
  println();
}

void Velleman_8056::println(int n)
{
  print(n);
  println();
}

void Velleman_8056::println(long n)
{
  print(n);
  println();  
}

void Velleman_8056::println(unsigned long n)
{
  print(n);
  println();  
}

void Velleman_8056::println(long n, int base)
{
  print(n, base);
  println();
}

// Private Methods /////////////////////////////////////////////////////////////

void Velleman_8056::printNumber(unsigned long n, uint8_t base)
{
  unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
  unsigned long i = 0;

  if (n == 0) {
    print('0');
    return;
  } 

  while (n > 0) {
    buf[i++] = n % base;
    n /= base;
  }

  for (; i > 0; i--)
    print((char) (buf[i - 1] < 10 ? '0' + buf[i - 1] : 'A' + buf[i - 1] - 10));
}



