 ;***************************************************************************
;
;	16F84A adapter for an 8 relay Velleman k8056 board
;	=====================================================
;
;	Author : Daniel CALLERAND
;	
;	Date : 01/11/2011
;
;	Current Status : working fine, but could be optimised
;
;	This piece of software is mostly NOT written by me.
;	 
;	I wrote the rs232 routine only. Other routine were 
;	sligthly adapted to suit my needs and were intially 
;	written by Peter Luethi.
;
;	Original Code from Peter Luethi Handle TX/RX and
;	is distributed as incude files
;
;	Here all is included into one source so that
;	it's easier to customise code, and we only handle
;	Rx. 
;
;	Compared to the original protocol of the k8056 board
;	we do not handle an adress board and we have no CRC
;
;	The only protocol is a byte with each bit symbolising
;	one relay on the board LSB -> Relay 1, MSB-> Relay 8  
;	
;
;	=====================================================
;	Former Notes
;	=====================================================
;	 
;	written by Peter Luethi, 26.03.1999, Dietikon, Switzerland
;	http://www.electronic-engineering.ch
;	last update: 16.01.2005
;
;	V1.02:	Fixed copy/paste issue of ISR context store/restore
;		(nobody is perfect): Erroneously erased INTCON,INTF
;		clearing, resulting in endless ISR calling...
;		Re-structured entire ISR and RS232 echo sub-routines
;		(11.04.2004)
;
;	V1.01:	ISR context restore improvements (21.11.1999)
;
;	V1.00:	Initial release (26.3.1999)
;
;	This code and accompanying files may be distributed freely and
;	modified, provided this header with my name and this notice remain
;	intact. Ownership rights remain with me.
;	You may not sell this software without my approval.
;
;	This software comes with no guarantee or warranty except for my
;	good intentions. By using this code you agree to indemnify me from
;	any liability that might arise from its use.
;
;
;	SPECIFICATIONS:
;	===============
;	Processor:			Microchip PIC 16F84A
;	Clock Frequency:		4.00 MHz XT
;	Throughput:			1 MIPS
;	Baud Rate:			adjustable inside this code by commenting/un commenting some lines
;
;
;	DESCRIPTION:
;	============
;	Developed and tested on PIC 16F84A, executeable on all interrupt
;	featured PICs.
;	Program RS232 reception sent by an arduino module, itself controlled with a PS2 keyboard
;	Aim is to drive some lights, or any kind on/off power source through a relay board
;		Reception (register RXD) through interrupts (PortB0 IRQ).
;
;***************************************************************************

;***** COMPILATION MESSAGES & WARNINGS *****

	ERRORLEVEL -207 	; found label after column 1
	ERRORLEVEL -302 	; register in operand not in bank 0

;***** PROCESSOR DECLARATION & CONFIGURATION *****

	PROCESSOR 16F84A
	#include "p16f84a.inc"
	
	; embed Configuration Data within .asm File.
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** MEMORY STRUCTURE *****

	ORG     0x00			; processor reset vector
  	goto    MAIN

	ORG     0x04			; interrupt vector
	goto	ISR			; Interrupt Service Routine (ISR)

;***** PORT DECLARATION *****

	#define	TXport	PORTA,0x00	; RS232 output port, could be
	#define	TXtris	TRISA,0x02	; any active push/pull port

;***** CONSTANT DECLARATION *****

	CONSTANT BASE = 0x0C		; base address of user file registers	

;***** REGISTER DECLARATION *****

	FLAGreg	equ	BASE+d'7'
	#define	RSflag	FLAGreg,0x00	; RS232 data reception flag

	TXD	equ	BASE+d'8'	; TX-Data register
	RXD	equ	BASE+d'9'	; RX-Data register

	W_TEMP	equ	BASE+d'10'	; context register (ISR)
	STATUS_TEMP equ	BASE+d'11'	; context register (ISR)
	PCLATH_TEMP equ	BASE+d'12'	; context register (ISR)
	FSR_TEMP equ	BASE+d'13'	; context register (ISR)

;***** INCLUDE FILES *****

	#include "..\modules\m_bank.asm"
	#include "..\modules\m_wait.asm"
;	#include "..\modules\m_rs192.asm"	; specify desired RS232 module

;***** RS232 ******

;***** INCLUDE FILES *****

	IFNDEF	M_BANK_ID
		ERROR "Missing include file: m_bank.asm"
	ENDIF

;***** HARDWARE DECLARATION *****

	IFNDEF	TXport
		ERROR "Define TXport in MAIN PROGRAM."
	ENDIF
	IFNDEF	TXtris
		ERROR "Define TXtris in MAIN PROGRAM."
	ENDIF
	
	MESSG "Default RS232 RXport is PORTB,0x00."

	#define	RXport	PORTB,0x00	; needs to be an interrupt supervised port
	#define	RXtris	TRISB,0x00	; if modified, set adequate flags in INTCON register
	
;***** CONSTANT DECLARATION *****

	CONSTANT NUM_BITS = d'8'		; number of data bits to transmit/receive


;Uncomment this section to switch to 19200
;19200
;	CONSTANT WAIT_GENERAL = 0x0C	; delay for transmission and reception
;	CONSTANT WAIT_RX_SB = 0x01		; delay for reception of start bit

;Uncomment this section to switch to 9600
;9600
	CONSTANT WAIT_GENERAL = 0x1D	; delay for transmission and reception
	CONSTANT WAIT_RX_SB = 0x08		; delay for reception of start bit


	CONSTANT LF =	d'10'		; line feed
	CONSTANT CR =	d'13'		; carriage return
	CONSTANT TAB =	d'9'		; tabulator
	CONSTANT BS =	d'8'		; backspace

;***** REGISTER DECLARATION *****

	IFNDEF	BASE
	  ERROR "Declare BASE (Base address of user file registers) in MAIN PROGRAM"
	ENDIF

	TEMP1	set	BASE+d'0'	; universal temporary register
	TEMP2	set	BASE+d'1'

	IFNDEF	TXD
	  ERROR "Declare TXD register in MAIN PROGRAM"
	ENDIF
	IFNDEF	RXD
	  ERROR "Declare RXD register in MAIN PROGRAM"
	ENDIF	

;***** MACROS *****

RS232init macro
	BANK1
	bcf	TXtris		; set output
	bsf	RXtris		; set input with weak pull-up
	bcf	OPTION_REG,INTEDG ; RS232 interrupt on falling edge
	BANK0
	bsf	TXport		; set default state: logical 1
	bcf	INTCON,INTF	; ensure interrupt flag is cleared
	bsf	INTCON,INTE	; enable RB0/INT interrupt
;	bsf	INTCON,GIE	; enable global interrupt
	endm

SEND	macro	S_string	; "SEND 'X'" sends character to RS232
	movlw	S_string
	call	SENDsub
	endm

SENDw	macro
	call	SENDsub
	endm

RECEIVE	macro
	call	SB_Wait		; first wait subroutine
	btfsc	RXport
	goto	_RSerror	; no valid start bit
	movlw	NUM_BITS	; ### number of data bits to receive, usually 8
	movwf	TEMP1
_RECa call	T_Wait		; inter-baud wait subroutine
	btfsc	RXport
	bsf		RXD,0x07
	btfss	RXport
	bcf		RXD,0x07
	decfsz	TEMP1,w		; skip if TEMP1 == 1
	rrf		RXD,f		; do this only NUM_BITS-1 times
	decfsz	TEMP1,f
	goto	_RECa
	call	T_Wait		; inter-baud wait subroutine
	btfss	RXport		; check if stop bit is valid
	goto	_RSerror	; no valid stop bit
	endm

;***** SUBROUTINES *****

SENDsub	movwf	TXD		; store in data register
	bcf		TXport		; start bit
	movlw	NUM_BITS	; ### number of data bits to send, usually 8
	movwf	TEMP1
	call	T_Wait
_SENDa	btfsc	TXD,0x00	; send LSB first
	bsf		TXport
	btfss	TXD,0x00
	bcf		TXport
	rrf		TXD,f
	call	T_Wait
	decfsz	TEMP1,f
	goto	_SENDa
	bsf		TXport		; stop bit
	call	T_Wait
	call	T_Wait		; allow some time for re-synchronization
	RETURN

T_Wait movlw WAIT_GENERAL	; ### for transmission & reception
	movwf	TEMP2		; total delay until next
	goto	X_Wait		; bit: 52 us @ 19200 baud

;*** When entering this subroutine, ISR context restore has already consumed some cycles ***
SB_Wait movlw WAIT_RX_SB	; ### for reception of start bit
	movwf	TEMP2		; total delay: 26 us @ 19200 baud
	;goto	X_Wait		; (=> sampling in the center of each bit)
	
X_Wait	decfsz	TEMP2,f		; wait loop
	goto	X_Wait
	RETURN

_RSerror clrf	RXD		; invalid data
	goto _ISR_RS232error	; goto RS232 error handling in ISR



;***** MACROS *****

;***** SUB-ROUTINES *****

RSservice 
	movfw	RXD			; get RS232 data

	btfss	RXD,0		;Test bit and jump to Stop if clear
	goto stop_RB1
	bsf		PORTB,3		;Otherwise turn on an output
	goto	test_RX1	;Go to next bit test

stop_RB1				
	bcf		PORTB,3		;Turn off an output

test_RX1

	btfss	RXD,1		;Test bit and jump to Stop if clear
	goto stop_RB2		;Test bit and jump to Stop if clear
	bsf		PORTB,2		;Otherwise turn on an output
	goto	test_RX2	;Go to next bit test		

stop_RB2
	bcf		PORTB,2		;Turn off an output

test_RX2

	btfss	RXD,2		;Test bit and jump to Stop if clear
	goto stop_RB3		;Test bit and jump to Stop if clear
	bsf		PORTB,4		;Otherwise turn on an output
	goto	test_RX3	;Go to next bit test

stop_RB3
	bcf		PORTB,4		;Turn off an output

test_RX3

	btfss	RXD,3		;Test bit and jump to Stop if clear
	goto stop_RB4		;Test bit and jump to Stop if clear
	bsf		PORTB,5		;Otherwise turn on an output
	goto	test_RX4	;Go to next bit test

stop_RB4
	bcf		PORTB,5		;Turn off an output

test_RX4

	btfss	RXD,4		;Test bit and jump to Stop if clear
	goto stop_RB5		;Test bit and jump to Stop if clear
	bsf		PORTB,6		;Otherwise turn on an output
	goto	test_RX5	;Go to next bit test

stop_RB5
	bcf		PORTB,6		;Turn off an output

test_RX5

	btfss	RXD,5		;Test bit and jump to Stop if clear
	goto stop_RB6		;Test bit and jump to Stop if clear
	bsf		PORTA,0		;Otherwise turn on an output
	goto	test_RX6	;Go to next bit test

stop_RB6
	bcf		PORTA,0		;Turn off an output

test_RX6

	btfss	RXD,6		;Test bit and jump to Stop if clear
	goto stop_RA0		;Test bit and jump to Stop if clear
	bsf		PORTA,1		;Otherwise turn on an output
	goto	test_RX7	;Go to next bit test

stop_RA0
	bcf		PORTA,1		;Turn off an output

test_RX7

	btfss	RXD,7		;Test bit and jump to Stop if clear
	goto stop_RA1		;Test bit and jump to Stop if clear
	bsf		PORTB,1		;Otherwise turn on an output
	goto	fin_test_RX	;Go to end of test

stop_RA1
	bcf		PORTB,1		;Turn off an output

fin_test_RX

	; end of RS232 service (echo & display)
	bcf	RSflag		; reset RS232 data reception flag
	bsf	INTCON,INTE	; re-enable RB0/INT interrupt
	RETURN

;***** INTERRUPT SERVICE ROUTINE *****

ISR	;************************
	;*** ISR CONTEXT SAVE ***
	;************************

	bcf	INTCON,GIE	; disable all interrupts
	btfsc	INTCON,GIE	; assure interrupts are disabled
	goto	ISR
	movwf	W_TEMP		; context save: W
	swapf	STATUS,W	; context save: STATUS
	movwf	STATUS_TEMP	; context save
	clrf	STATUS		; bank 0, regardless of current bank
	movfw	PCLATH		; context save: PCLATH
	movwf	PCLATH_TEMP	; context save
	clrf	PCLATH		; page zero, regardless of current page
	bcf	STATUS,IRP	; return to bank 0
	movfw	FSR		; context save: FSR
	movwf	FSR_TEMP	; context save
	;*** context save done ***

	;**************************
	;*** ISR MAIN EXECUTION ***
	;**************************
	
	;*** check origin of interrupt ***
	btfsc	INTCON,INTF	; check for RB0/INT interrupt
	goto	_ISR_RS232	; if set, there was a keypad stroke

	btfsc	INTCON,RBIF	;Tester si interrupt RB4/7
	goto	_ISR_SWTEST	; if set, the button was pushed

	; catch-all
	goto	ISRend		; unexpected IRQ, terminate execution of ISR

	;******************************
	;*** RS232 DATA ACQUISITION ***
	;******************************
_ISR_RS232
	; first, disable interrupt source
	bcf	INTCON,INTE	; disable RB0/INT interrupt
	; second, acquire RS232 data
	;RECEIVE			; macro of RS232 software reception

	bsf PORTA,2			;Led Receive On
	
	;Debug
	call	SB_Wait		; first wait subroutine
	btfsc	RXport
	goto	_RSerror	; no valid start bit
	movlw	NUM_BITS	; ### number of data bits to receive, usually 8
	movwf	TEMP1
_RECa call	T_Wait		; inter-baud wait subroutine
	btfsc	RXport
	bsf		RXD,0x07
	btfss	RXport
	bcf		RXD,0x07
	decfsz	TEMP1,w		; skip if TEMP1 == 1
	rrf		RXD,f		; do this only NUM_BITS-1 times
	decfsz	TEMP1,f
	goto	_RECa
	call	T_Wait		; inter-baud wait subroutine
	btfss	RXport		; check if stop bit is valid
	goto	_RSerror	; no valid stop bit

	;Fin debug

	bsf	RSflag		; enable RS232 data reception flag

	bcf PORTA,2			;Led Receive Off

	goto	_ISR_RS232end	; terminate RS232 ISR properly

	;***********************************
	;*** CLEARING OF INTERRUPT FLAGS ***
	;***********************************
	; NOTE: Below, I only clear the interrupt flags! This does not
	; necessarily mean, that the interrupts are already re-enabled.
	; Basically, interrupt re-enabling is carried out at the end of
	; the corresponding service routine in normal operation mode.
	; The flag responsible for the current ISR call has to be cleared
	; to prevent recursive ISR calls. Other interrupt flags, activated
	; during execution of this ISR, will immediately be served upon
	; termination of the current ISR run.
_ISR_RS232error
	bsf	INTCON,INTE	; after error, re-enable IRQ already here
_ISR_RS232end
	bcf	INTCON,INTF	; clear RB0/INT interrupt flag
	goto	ISRend		; terminate execution of ISR

	;******************************
	;*** TEST BUTTON		    ***
	;******************************

_ISR_SWTEST	

	bcf		INTCON,RBIE	; disable RB4-7/INT interrupt
	btfsc	PORTB,4		;
	goto	ISRend1 
	bsf		PORTB,4			;

	
	movlw	b'00011111'			; prparer pour inversion bit
	BANK0
	xorwf	PORTA,f			; inverser LED

ISRend1
	bcf	INTCON,RBIF				; Effacer flag interruption
	bsf	INTCON,RBIE				; enable RB4-7/INT interrupt
	goto	ISRend		; terminate execution of ISR

	;*****************************************
	;*** ISR TERMINATION (CONTEXT RESTORE) ***
	;*****************************************

ISRend	movfw	FSR_TEMP	; context restore
	movwf	FSR		; context restore
	movfw	PCLATH_TEMP	; context restore
	movwf	PCLATH		; context restore
	swapf	STATUS_TEMP,W	; context restore
	movwf	STATUS		; context restore
	swapf	W_TEMP,F	; context restore
	swapf	W_TEMP,W	; context restore
	RETFIE			; enable global interrupt (INTCON,GIE)

;***** END OF INTERRUPT SERVICE ROUTINE *****


;************** MAIN **************

MAIN	
	clrf	INTCON		; reset interrupts (disable all)	
	RS232init		; RS232 initialization
	BANK1
	bcf TRISB,1		; led en sortie
	bcf TRISB,2		; led en sortie
	bcf TRISB,3		; led en sortie
	bcf TRISB,4		; led en sortie
	bcf TRISB,5		; led en sortie
	bcf TRISB,6		; led en sortie
	
	movlw 0x00
	movwf PORTA

	BANK0		

	movlw 0x00
	movwf PORTA

	
	bcf PORTB,1
	bcf PORTB,2
	bcf PORTB,3
	bcf PORTB,4
	bcf PORTB,5
	bcf PORTB,6

	;bsf	INTCON,RBIE ; activation interruption RB4/7
	;bsf PORTB,4		; Mise a 1 de la pin du switch de test
	bsf	INTCON,GIE	; enable global interrupt

	clrf	FLAGreg		; initialize all flags

LOOP	btfsc	RSflag		; check RS232 data reception flag
	call	RSservice	; if set, call RS232 echo & LCD display routine
	goto	LOOP

	END
